@@ -20,6 +20,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <Python.h>
 #include "ex.h"
 #include "symbol.h"
 #include "add.h"
@@ -320,18 +321,17 @@ bool ex::is_integer_one() const
 {
     if (!is_exactly_a<numeric>(*this))
         return false;
-    return ex_to<numeric>(*this).is_integer()
-        and ex_to<numeric>(*this).is_one();
+    const numeric& num = ex_to<numeric>(*this);
+    return num.is_integer() and num.is_one();
 }
   
 bool ex::is_integer_pmone() const
 {
     if (!is_exactly_a<numeric>(*this))
         return false;
-    numeric num = ex_to<numeric>(*this);
-    return ((ex_to<numeric>(*this).is_integer())
-        and (ex_to<numeric>(*this).is_one()
-            or ex_to<numeric>(*this).is_minus_one()));
+    const numeric& num = ex_to<numeric>(*this);
+    return num.is_integer()
+           and (num.is_one() or num.is_minus_one());
 }
 
 void ex::set_domain(unsigned d)
@@ -402,6 +402,7 @@ static void collect_bound_symbols(const ex& e, symbolset& syms)
 {
         static unsigned int sum_serial = function::find_function("sum", 4);
         static unsigned int integral_serial = function::find_function("integrate", 4);
+        static unsigned int limit_serial = function::find_function("limit", 0);
 	if (is_exactly_a<function>(e)) {
                 const function& f = ex_to<function>(e);
                 if (f.get_serial() == sum_serial
@@ -414,6 +415,11 @@ static void collect_bound_symbols(const ex& e, symbolset& syms)
                         syms.insert(ex_to<symbol>(f.op(1)));
                         return collect_bound_symbols(f.op(0), syms);
                 }
+                if (f.get_serial() == limit_serial
+                    and is_exactly_a<symbol>(f.op(1))) {
+                        syms.insert(ex_to<symbol>(f.op(1)));
+                        return collect_bound_symbols(f.op(0), syms);
+                }
         }
         else if (is_exactly_a<fderivative>(e)) {
                 const fderivative& d = ex_to<fderivative>(e);
@@ -448,6 +454,23 @@ ex ex::sorted_op(size_t i) const
 		return bp->op(i);
 }
 
+static bool has_nonposint_power(const ex& x, const symbol& symb)
+{
+        if (is_exactly_a<power>(x)) {
+                const power& p = ex_to<power>(x);
+                if (is_exactly_a<add>(p.op(0))
+                    and has_symbol(p.op(0), symb)
+                    and (not is_exactly_a<numeric>(p.op(1))
+                    or not ex_to<numeric>(p.op(1)).is_pos_integer()))
+                return true;
+        }
+        for (size_t i=0; i<x.nops(); ++i)
+        if (has_nonposint_power(x.op(i), symb))
+                return true;
+
+	return false;
+}
+
 // Helper function: Return True if term is a monomial in symb.
 // If so, fill vec (applying map at the same time) with (coeff, expo) pairs.
 static bool match_monom(const ex& term, const symbol& symb,
@@ -554,6 +577,11 @@ void ex::coefficients(const ex & s, expairvec & vec) const
         exmap submap {{s, symb}}, revmap {{symb, s}};
         ex sub = subs(submap);
 
+        if (has_nonposint_power(sub, symb)) {
+                vec.push_back(std::make_pair(*this, _ex0));
+                return;
+        }
+
         if (is_exactly_a<add>(sub)) {
                 const add& addref = ex_to<add>(sub);
                 const ex& oc = addref.op(addref.nops()+1);
@@ -881,43 +909,6 @@ basic & ex::construct_from_long(long i)
 	}
 }
 	
-basic & ex::construct_from_ulong(unsigned long i)
-{
-	switch (i) {  // prefer flyweights over new objects
-	case 0:
-		return *const_cast<numeric *>(_num0_p);
-	case 1:
-		return *const_cast<numeric *>(_num1_p);
-	case 2:
-		return *const_cast<numeric *>(_num2_p);
-	case 3:
-		return *const_cast<numeric *>(_num3_p);
-	case 4:
-		return *const_cast<numeric *>(_num4_p);
-	case 5:
-		return *const_cast<numeric *>(_num5_p);
-	case 6:
-		return *const_cast<numeric *>(_num6_p);
-	case 7:
-		return *const_cast<numeric *>(_num7_p);
-	case 8:
-		return *const_cast<numeric *>(_num8_p);
-	case 9:
-		return *const_cast<numeric *>(_num9_p);
-	case 10:
-		return *const_cast<numeric *>(_num10_p);
-	case 11:
-		return *const_cast<numeric *>(_num11_p);
-	case 12:
-		return *const_cast<numeric *>(_num12_p);
-	default:
-		basic *bp = new numeric(i);
-		bp->setflag(status_flags::dynallocated);
-		GINAC_ASSERT(bp->get_refcount() == 0);
-		return *bp;
-	}
-}
-	
 basic & ex::construct_from_double(double d)
 {
 	basic *bp = new numeric(d);
