@@ -122,19 +122,19 @@ expairseq::expairseq(const exvector &v) : inherited(&expairseq::tinfo_static)
 	GINAC_ASSERT(is_canonical());
 }
 
-expairseq::expairseq(const epvector &v, ex oc, bool do_index_renaming)
-  : inherited(&expairseq::tinfo_static), overall_coeff(std::move(oc))
+expairseq::expairseq(const epvector &v,
+                const numeric& oc, bool do_index_renaming)
+  : inherited(&expairseq::tinfo_static), overall_coeff(oc)
 {
-	GINAC_ASSERT(is_a<numeric>(oc));
 	construct_from_epvector(v, do_index_renaming);
 	GINAC_ASSERT(is_canonical());
 }
 
-expairseq::expairseq(std::unique_ptr<epvector> vp, ex oc, bool do_index_renaming)
-  : inherited(&expairseq::tinfo_static), overall_coeff(std::move(oc))
+expairseq::expairseq(std::unique_ptr<epvector> vp,
+                const numeric& oc, bool do_index_renaming)
+  : inherited(&expairseq::tinfo_static), overall_coeff(oc)
 {
 	GINAC_ASSERT(vp.get()!=0);
-	GINAC_ASSERT(is_a<numeric>(oc));
 	construct_from_epvector(*vp, do_index_renaming);
 	GINAC_ASSERT(is_canonical());
 }
@@ -161,7 +161,9 @@ expairseq::expairseq(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst
 		seq.push_back(expair(lrest, lcoeff));
 	}
 
-	n.find_ex("overall_coeff", overall_coeff, sym_lst);
+        ex oc;
+	n.find_ex("overall_coeff", oc, sym_lst);
+        overall_coeff = ex_to<numeric>(oc);
 
 	canonicalize();
 	GINAC_ASSERT(is_canonical());
@@ -174,7 +176,7 @@ void expairseq::archive(archive_node &n) const
 		n.add_ex("rest", elem.rest);
 		n.add_ex("coeff", elem.coeff);
 	}
-	n.add_ex("overall_coeff", overall_coeff);
+	n.add_ex("overall_coeff", ex(overall_coeff));
 }
 
 DEFAULT_UNARCHIVE(expairseq)
@@ -329,7 +331,8 @@ ex expairseq::map(map_function &f) const
 	else {
 		ex newcoeff = f(overall_coeff);
 		if(is_exactly_a<numeric>(newcoeff))
-			return thisexpairseq(std::move(v), newcoeff, true);
+			return thisexpairseq(std::move(v),
+                                        ex_to<numeric>(newcoeff), true);
 		else {
 			v->push_back(split_ex_to_pair(newcoeff));
 			return thisexpairseq(std::move(v), default_overall_coeff(), true);
@@ -377,8 +380,9 @@ epvector* conjugateepvector(const epvector& epv)
 ex expairseq::conjugate() const
 {
 	std::unique_ptr<epvector> newepv(conjugateepvector(seq));
-	ex x = overall_coeff.conjugate();
-	if (!newepv && are_ex_trivially_equal(x, overall_coeff)) {
+        // this is known to be numeric
+	const numeric& x = ex_to<numeric>(overall_coeff.conjugate());
+	if (!newepv and overall_coeff.is_equal(x)) {
 		return *this;
 	}
 	ex result = thisexpairseq(newepv ? *newepv : seq, x);
@@ -518,7 +522,7 @@ int expairseq::compare_same_type(const basic &other) const
 		return (seq.size()<o.seq.size()) ? -1 : 1;
 
 	// compare overall_coeff
-	cmpval = overall_coeff.compare(o.overall_coeff);
+	cmpval = overall_coeff.compare_same_type(o.overall_coeff);
 	if (cmpval!=0)
 		return cmpval;
 
@@ -689,12 +693,12 @@ ex expairseq::expand(unsigned options) const
  *  ctor because the name (add, mul,...) is unknown on the expaiseq level.  In
  *  order for this trick to work a derived class must of course override this
  *  definition. */
-ex expairseq::thisexpairseq(const epvector &v, const ex &oc, bool do_index_renaming) const
+ex expairseq::thisexpairseq(const epvector &v, const numeric &oc, bool do_index_renaming) const
 {
 	return expairseq(v, oc, do_index_renaming);
 }
 
-ex expairseq::thisexpairseq(std::unique_ptr<epvector> vp, const ex &oc, bool do_index_renaming) const
+ex expairseq::thisexpairseq(std::unique_ptr<epvector> vp, const numeric &oc, bool do_index_renaming) const
 {
 	return expairseq(std::move(vp), oc, do_index_renaming);
 }
@@ -740,21 +744,18 @@ expair expairseq::split_ex_to_pair(const ex &e) const
 
 
 expair expairseq::combine_ex_with_coeff_to_pair(const ex &e,
-                                                const ex &c) const
+                                                const numeric &c) const
 {
-	GINAC_ASSERT(is_exactly_a<numeric>(c));
-	
 	return expair(e,c);
 }
 
 
 expair expairseq::combine_pair_with_coeff_to_pair(const expair &p,
-                                                  const ex &c) const
+                                                  const numeric &c) const
 {
 	GINAC_ASSERT(is_exactly_a<numeric>(p.coeff));
-	GINAC_ASSERT(is_exactly_a<numeric>(c));
 	
-	return expair(p.rest,ex_to<numeric>(p.coeff).mul_dyn(ex_to<numeric>(c)));
+	return expair(p.rest,ex_to<numeric>(p.coeff).mul_dyn(c));
 }
 
 
@@ -773,31 +774,25 @@ bool expairseq::expair_needs_further_processing(epp /*unused*/)
 	return false;
 }
 
-ex expairseq::default_overall_coeff() const
+numeric expairseq::default_overall_coeff() const
 {
-	return _ex0;
+	return *_num0_p;
 }
 
 bool expairseq::overall_coeff_equals_default() const
 {
-    numeric num = ex_to<numeric>(overall_coeff);
-    return (num.is_integer() && num.is_equal(ex_to<numeric>(default_overall_coeff())));
+    return (overall_coeff.is_exact()
+         and overall_coeff.is_equal(default_overall_coeff()));
 }
 
-void expairseq::combine_overall_coeff(const ex &c)
+void expairseq::combine_overall_coeff(const numeric &c)
 {
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	GINAC_ASSERT(is_exactly_a<numeric>(c));
-	overall_coeff = ex_to<numeric>(overall_coeff).add_dyn(ex_to<numeric>(c));
+         overall_coeff += c;
 }
 
-void expairseq::combine_overall_coeff(const ex &c1, const ex &c2)
+void expairseq::combine_overall_coeff(const numeric &c1, const numeric &c2)
 {
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	GINAC_ASSERT(is_exactly_a<numeric>(c1));
-	GINAC_ASSERT(is_exactly_a<numeric>(c2));
-	overall_coeff = ex_to<numeric>(overall_coeff).
-	                add_dyn(ex_to<numeric>(c1).mul(ex_to<numeric>(c2)));
+	overall_coeff += c1.mul(c2);
 }
 
 bool expairseq::can_make_flat(const expair & /*unused*/) const
@@ -830,9 +825,9 @@ void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 #if EXPAIRSEQ_USE_HASHTAB
 			unsigned totalsize = ex_to<expairseq>(lh).seq.size() +
 			                     ex_to<expairseq>(rh).seq.size();
-			if (calc_hashtabsize(totalsize)!=0) {
-				construct_from_2_ex_via_exvector(lh,rh);
-			} else {
+                        construct_from_2_ex_via_exvector(lh,rh);
+                        {
+                } else {
 #endif // EXPAIRSEQ_USE_HASHTAB
 				if (is_exactly_a<mul>(lh) && lh.info(info_flags::has_indices) && 
 					rh.info(info_flags::has_indices)) {
@@ -883,23 +878,25 @@ void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 #endif // EXPAIRSEQ_USE_HASHTAB
 	
 	if (is_exactly_a<numeric>(lh)) {
+                const numeric& lhn = ex_to<numeric>(lh);
 		if (is_exactly_a<numeric>(rh)) {
-			combine_overall_coeff(lh);
-			combine_overall_coeff(rh);
+			combine_overall_coeff(lhn);
+			combine_overall_coeff(ex_to<numeric>(rh));
 		} else {
-			combine_overall_coeff(lh);
+			combine_overall_coeff(lhn);
 			seq.push_back(split_ex_to_pair(rh));
 		}
 	} else {
 		if (is_exactly_a<numeric>(rh)) {
-			combine_overall_coeff(rh);
+			combine_overall_coeff(ex_to<numeric>(rh));
 			seq.push_back(split_ex_to_pair(lh));
 		} else {
 			expair p1 = split_ex_to_pair(lh);
 			expair p2 = split_ex_to_pair(rh);
 
 			int cmpval = p1.rest.compare(p2.rest);
-			if (cmpval==0 && unlikely(!is_exactly_a<infinity>(p1.rest))) {
+			if (cmpval==0
+                            and likely(not is_exactly_a<infinity>(p1.rest))) {
 				p1.coeff = ex_to<numeric>(p1.coeff).add_dyn(ex_to<numeric>(p2.coeff));
 				if (!ex_to<numeric>(p1.coeff).is_zero()) {
 
@@ -988,7 +985,7 @@ void expairseq::construct_from_expairseq_ex(const expairseq &s,
 {
 	combine_overall_coeff(s.overall_coeff);
 	if (is_exactly_a<numeric>(e)) {
-		combine_overall_coeff(e);
+		combine_overall_coeff(ex_to<numeric>(e));
 		seq = s.seq;
 		return;
 	}
@@ -1123,7 +1120,7 @@ void expairseq::make_flat(const exvector &v, bool do_hold)
 				seq.push_back(elem2);
 		} else {
 			if (is_exactly_a<numeric>(elem))
-				combine_overall_coeff(elem);
+				combine_overall_coeff(ex_to<numeric>(elem));
 			else {
 				ex newfactor = mf.handle_factor(elem, _ex1);
 				seq.push_back(split_ex_to_pair(newfactor));
@@ -1164,19 +1161,19 @@ void expairseq::make_flat(const epvector &v, bool do_index_renaming)
 		    this->can_make_flat(elem)) {
 			ex newrest = mf.handle_factor(elem.rest, elem.coeff);
 			const expairseq &subseqref = ex_to<expairseq>(newrest);
-			combine_overall_coeff(ex_to<numeric>(subseqref.overall_coeff),
-			                                    ex_to<numeric>(elem.coeff));
+			combine_overall_coeff(subseqref.overall_coeff,
+			                      ex_to<numeric>(elem.coeff));
                         for (const auto & elem2 : subseqref.seq)
 				seq.push_back(expair(elem2.rest,
                                         ex_to<numeric>(elem2.coeff).mul_dyn(ex_to<numeric>(elem.coeff))));
 		} else {
 			if (elem.is_canonical_numeric())
-				combine_overall_coeff(mf.handle_factor(elem.rest, _ex1));
+				combine_overall_coeff(ex_to<numeric>(mf.handle_factor(elem.rest, _ex1)));
 			else {
 				const ex& rest = elem.rest;
 				const ex& newrest = mf.handle_factor(rest, elem.coeff);
                                 if (newrest.is_zero())
-                                        combine_overall_coeff(newrest);
+                                        combine_overall_coeff(*_num0_p);
 				else if (are_ex_trivially_equal(newrest, rest))
 					seq.push_back(elem);
 				else
@@ -1211,8 +1208,8 @@ void expairseq::combine_same_terms_sorted_seq()
 	// possible from then on the sequence has changed and must be compacted
 	bool must_copy = false;
 	while (itin2!=last) {
-		if (itin1->rest.compare(itin2->rest)==0 &&
-				unlikely(!is_exactly_a<infinity>(itin1->rest))) {
+		if (itin1->rest.compare(itin2->rest)==0
+                    and likely(not is_exactly_a<infinity>(itin1->rest))) {
 			itin1->coeff = ex_to<numeric>(itin1->coeff).
                                 add_dyn(ex_to<numeric>(itin2->coeff));
 			if (expair_needs_further_processing(itin1))
@@ -1620,7 +1617,7 @@ std::unique_ptr<epvector> expairseq::expandchildren(unsigned options) const
 				                         cit->coeff));
 				++cit;
 			}
-			return std::move(s);
+			return s;
 		}
 
 		++cit;
@@ -1644,7 +1641,7 @@ std::unique_ptr<epvector> expairseq::evalchildren(int level) const
 	auto cit = seq.begin();
 	while (cit!=last) {
                 const ex evaled_rest = level==1 ? cit->rest : cit->rest.eval(level-1);
-                const expair evaled_pair = combine_ex_with_coeff_to_pair(evaled_rest, cit->coeff);
+                const expair evaled_pair = combine_ex_with_coeff_to_pair(evaled_rest, ex_to<numeric>(cit->coeff));
                 if (unlikely(!evaled_pair.is_equal(*cit))) {
 
 			// something changed: copy seq, eval, and return it
@@ -1662,10 +1659,10 @@ std::unique_ptr<epvector> expairseq::evalchildren(int level) const
 			while (cit != last) {
                                 const ex evaled_rest1 = level==1 ? cit->rest : cit->rest.eval(level-1);
 				s->push_back(combine_ex_with_coeff_to_pair(evaled_rest1,
-				                                       cit->coeff));
+				             ex_to<numeric>(cit->coeff)));
 				++cit;
 			}
-			return std::move(s);
+			return s;
 		}
 		++cit;
 	}
@@ -1722,7 +1719,7 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 					s->push_back(split_ex_to_pair(recombine_pair_to_ex(*cit).subs(m, options)));
 					++cit;
 				}
-				return std::move(s);
+				return s;
 			}
 
 			++cit;
@@ -1744,15 +1741,16 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 				s->insert(s->begin(), seq.begin(), cit);
 			
 				// Copy first changed element
-				s->push_back(combine_ex_with_coeff_to_pair(subsed_ex, cit->coeff));
+				s->push_back(combine_ex_with_coeff_to_pair(subsed_ex,
+                                                        ex_to<numeric>(cit->coeff)));
 				++cit;
 
 				// Copy rest
 				while (cit != last) {
-					s->push_back(combine_ex_with_coeff_to_pair(cit->rest.subs(m, options), cit->coeff));
+					s->push_back(combine_ex_with_coeff_to_pair(cit->rest.subs(m, options), ex_to<numeric>(cit->coeff)));
 					++cit;
 				}
-				return std::move(s);
+				return s;
 			}
 
 			++cit;
