@@ -103,10 +103,9 @@ numeric add::integer_content() const
 		l = lcm(ex_to<numeric>(it->coeff).denom(), l);
 		it++;
 	}
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	c = gcd(ex_to<numeric>(overall_coeff).numer(), c);
-	l = lcm(ex_to<numeric>(overall_coeff).denom(), l);
-	return c/l;
+	c = gcd(overall_coeff.numer(), c);
+	l = lcm(overall_coeff.denom(), l);
+	return (c/l).abs();
 }
 
 numeric mul::integer_content() const
@@ -119,8 +118,7 @@ numeric mul::integer_content() const
 		++it;
 	}
 #endif // def DO_GINAC_ASSERT
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	return abs(ex_to<numeric>(overall_coeff));
+	return overall_coeff.abs();
 }
 
 
@@ -145,17 +143,17 @@ typedef std::map<ex2, exbool, ex2_less> ex2_exbool_remember;
 
 
 /** Return maximum (absolute value) coefficient of a polynomial.
- *  This function is used internally by heur_gcd().
+ *  This function was used internally by heur_gcd().
  *
  *  @return maximum coefficient
- *  @see heur_gcd */
+ */
 numeric ex::max_coefficient() const
 {
 	return bp->max_coefficient();
 }
 
 /** Implementation ex::max_coefficient().
- *  @see heur_gcd */
+ */
 numeric basic::max_coefficient() const
 {
 	return *_num1_p;
@@ -170,8 +168,7 @@ numeric add::max_coefficient() const
 {
 	auto it = seq.begin();
 	auto itend = seq.end();
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	numeric cur_max = abs(ex_to<numeric>(overall_coeff));
+	numeric cur_max = abs(overall_coeff);
 	while (it != itend) {
 		numeric a;
 		GINAC_ASSERT(!is_exactly_a<numeric>(it->rest));
@@ -193,8 +190,7 @@ numeric mul::max_coefficient() const
 		it++;
 	}
 #endif // def DO_GINAC_ASSERT
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	return abs(ex_to<numeric>(overall_coeff));
+	return abs(overall_coeff);
 }
 
 bool ex::is_linear(const symbol& x, ex& a, ex& b) const
@@ -228,11 +224,11 @@ bool ex::is_quadratic(const symbol& x, ex& a, ex& b, ex& c) const
 
 
 /** Apply symmetric modular homomorphism to an expanded multivariate
- *  polynomial.  This function is usually used internally by heur_gcd().
+ *  polynomial.  This function was usually used internally by heur_gcd().
  *
  *  @param xi  modulus
  *  @return mapped polynomial
- *  @see heur_gcd */
+ */
 ex basic::smod(const numeric &xi) const
 {
 	return *this;
@@ -256,8 +252,7 @@ ex add::smod(const numeric &xi) const
 			newseq.push_back(expair(it->rest, num_coeff));
 		it++;
 	}
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	numeric num_coeff = GiNaC::smod(ex_to<numeric>(overall_coeff), xi);
+	numeric num_coeff = GiNaC::smod(overall_coeff, xi);
 	return (new add(newseq, num_coeff))->setflag(status_flags::dynallocated);
 }
 
@@ -272,8 +267,7 @@ ex mul::smod(const numeric &xi) const
 	}
 #endif // def DO_GINAC_ASSERT
 	auto  mulcopyp = new mul(*this);
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	mulcopyp->overall_coeff = GiNaC::smod(ex_to<numeric>(overall_coeff),xi);
+	mulcopyp->overall_coeff = GiNaC::smod(overall_coeff,xi);
 	mulcopyp->clearflag(status_flags::evaluated);
 	mulcopyp->clearflag(status_flags::hash_calculated);
 	return mulcopyp->setflag(status_flags::dynallocated);
@@ -481,7 +475,7 @@ ex add::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options) co
 		dens.push_back(n.op(1));
 		it++;
 	}
-	ex n = ex_to<numeric>(overall_coeff).normal(repl, rev_lookup, level-1);
+	ex n = overall_coeff.normal(repl, rev_lookup, level-1);
 	nums.push_back(n.op(0));
 	dens.push_back(n.op(1));
 	GINAC_ASSERT(nums.size() == dens.size());
@@ -542,7 +536,7 @@ ex mul::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options) co
 		den.push_back(n.op(1));
 		it++;
 	}
-	n = ex_to<numeric>(overall_coeff).normal(repl, rev_lookup, level-1);
+	n = overall_coeff.normal(repl, rev_lookup, level-1);
 	num.push_back(n.op(0));
 	den.push_back(n.op(1));
 
