@@ -493,6 +493,12 @@ ex power::eval(int level) const
 	if (is_exactly_a<numeric>(ebasis)) {
 		basis_is_numerical = true;
 		num_basis = ex_to<numeric>(ebasis);
+
+                // (negative num)^even --> (positive num)^even
+                if (num_basis.is_negative()
+                    and eexponent.info(info_flags::even)
+                    and eexponent.info(info_flags::real))
+                        return power(-num_basis, eexponent);
 	}
 	if (is_exactly_a<numeric>(eexponent)) {
 		exponent_is_numerical = true;
@@ -552,7 +558,7 @@ ex power::eval(int level) const
 		return ebasis;
 
 	// ^(0,c1) -> 0 or exception  (depending on real value of c1)
-	if (ebasis.is_zero() && exponent_is_numerical) {
+	if (exponent_is_numerical and ebasis.is_zero() ) {
 		if ((num_exponent.real()).is_zero())
 			throw (std::domain_error("power::eval(): pow(0,I) is undefined"));
 		else if ((num_exponent.real()).is_negative())
@@ -577,6 +583,12 @@ ex power::eval(int level) const
                                           and ebasis.op(1).info(info_flags::real))))
 		return power(ebasis.op(0), ebasis.op(1) * eexponent);
 
+        // (negative oc)^even --> (positive oc)^even
+        if (is_exactly_a<mul>(ebasis)
+            and ex_to<mul>(ebasis).overall_coeff.is_negative()
+            and eexponent.info(info_flags::even)
+            and eexponent.info(info_flags::real))
+                return power(-ebasis, eexponent);
 
 	if (exponent_is_numerical) {
 		// ^(c1,c2) -> c1^c2  (c1, c2 numeric(),
@@ -616,7 +628,7 @@ ex power::eval(int level) const
 				}
 				if (q.is_zero()) {  // the exponent was in the allowed range 0<(n/m)<1
 					if (num_basis.is_rational()) {
-						bool ppower_equals_one; // = PyObject_IsTrue(PyTuple_GetItem(restuple, 2)) != 0;
+						bool ppower_equals_one;
                                                 numeric newbasis, ppower;
                                                 rational_power_parts(num_basis,
                                                                 num_exponent, 
@@ -659,11 +671,15 @@ ex power::eval(int level) const
 						|| (num_sub_exponent == *_num_1_p && num_exponent.is_positive())) {
 					return power(sub_basis,num_sub_exponent.mul(num_exponent));
 				}
+                                numeric pexp = num_sub_exponent * num_exponent;
+                                if (pexp.is_integer()
+                                    and num_sub_exponent.is_even()
+                                    and sub_basis.info(info_flags::real))
+                                        return power(abs(sub_basis), pexp);
 			}
 		}
 	
 		if (num_exponent.is_integer()) {
-                        
                         // ^(*(x,y,z),c1) -> *(x^c1,y^c1,z^c1) (c1 integer)
                         if (is_exactly_a<mul>(ebasis)) {
                                 return expand_mul(ex_to<mul>(ebasis), num_exponent, 0);
@@ -677,25 +693,28 @@ ex power::eval(int level) const
                                                         lead_coeff()).div(icont);
 
                                 const bool canonicalizable = lead_coeff.is_integer();
-                                const bool unit_normal = lead_coeff.is_pos_integer() || lead_coeff.is_parent_pos_char();
-                                if (canonicalizable && (! unit_normal))
-                                        icont = icont.mul(*_num_1_p);
+                                const add& addref = ex_to<add>(ebasis);
+                                const bool unit_normal = lead_coeff.is_positive() || lead_coeff.is_parent_pos_char();
+                                if (canonicalizable
+                                    and (! unit_normal)
+                                    and num_exponent.denom().is_one())
+                                        icont = icont.negative();
 
-                                if (canonicalizable and not icont.is_one()) {
-                                        const add& addref = ex_to<add>(ebasis);
+                                if (canonicalizable
+                                    and not icont.is_one()) {
                                         auto  addp = new add(addref);
                                         addp->setflag(status_flags::dynallocated);
                                         addp->clearflag(status_flags::hash_calculated);
-                                        addp->overall_coeff = ex_to<numeric>(addp->overall_coeff).div_dyn(icont);
+                                        addp->overall_coeff = addp->overall_coeff.div_dyn(icont);
                                         addp->seq_sorted.resize(0);
                                         for (auto & elem : addp->seq)
                                                 elem.coeff = ex_to<numeric>(elem.coeff).div_dyn(icont);
-
-                                        const numeric c = icont.pow_intexp(num_exponent);
+                                        const numeric c = icont.pow_intexp(num_exponent.to_long());
                                         if (likely(not c.is_one()))
-                                                return (new mul(power(*addp, num_exponent), c))->setflag(status_flags::dynallocated);
-                                        else
-                                                return power(*addp, num_exponent);
+                                                return (new mul(power(*addp, num_exponent), c))->setflag(status_flags::dynallocated |
+	                                               status_flags::evaluated);
+                                        return (new power(*addp, eexponent))->setflag(status_flags::dynallocated |
+	                                               status_flags::evaluated);
                                 }
                         }
                 }
@@ -705,12 +724,12 @@ ex power::eval(int level) const
 		if (is_exactly_a<mul>(ebasis)) {
 			GINAC_ASSERT(!num_exponent.is_integer()); // should have been handled above
 			const mul & mulref = ex_to<mul>(ebasis);
-			if (!mulref.overall_coeff.is_equal(_ex1)) {
-				const numeric & num_coeff = ex_to<numeric>(mulref.overall_coeff);
+			if (not mulref.overall_coeff.is_one()) {
+				const numeric & num_coeff = mulref.overall_coeff;
 				if (num_coeff.is_real()) {
 					if (num_coeff.is_positive()) {
 						auto mulp = new mul(mulref);
-						mulp->overall_coeff = _ex1;
+						mulp->overall_coeff = *_num1_p;
 						mulp->setflag(status_flags::dynallocated);
 						mulp->clearflag(status_flags::evaluated);
 						mulp->clearflag(status_flags::hash_calculated);
@@ -718,10 +737,10 @@ ex power::eval(int level) const
 						return (new mul(power(*mulp,exponent),
 						                power(num_coeff, num_exponent)))->setflag(status_flags::dynallocated);
 					} else {
-						GINAC_ASSERT(num_coeff.compare(*_num0_p)<0);
-						if (!num_coeff.is_equal(*_num_1_p)) {
+						GINAC_ASSERT(num_coeff.is_negative());
+						if (not num_coeff.is_minus_one()) {
 							auto mulp = new mul(mulref);
-							mulp->overall_coeff = _ex_1;
+							mulp->overall_coeff = *_num_1_p;
 							mulp->setflag(status_flags::dynallocated);
 							mulp->clearflag(status_flags::evaluated);
 							mulp->clearflag(status_flags::hash_calculated);
@@ -746,12 +765,6 @@ ex power::eval(int level) const
 					return exp(log(basis)*exponent);
 	}
 	
-	// negative^even --> positive^even
-        if (eexponent.info(info_flags::even)
-            and eexponent.info(info_flags::real)
-            and (-ebasis).info(info_flags::positive))
-                return power(-ebasis, eexponent);
-
 	if (are_ex_trivially_equal(ebasis,basis) &&
 	    are_ex_trivially_equal(eexponent,exponent)) {
 		return this->hold();
@@ -928,7 +941,7 @@ ex power::derivative(const symbol & s) const
 		newseq.reserve(2);
 		newseq.push_back(expair(basis, exponent - _ex1));
 		newseq.push_back(expair(basis.diff(s), _ex1));
-		return mul(newseq, exponent);
+		return mul(newseq, ex_to<numeric>(exponent));
 	} else {
 	    // If the exponent is not a function of s, we have the following nice
 	    // looking formula.  We use this to avoid getting ugly and hard to
@@ -1000,10 +1013,11 @@ ex power::expand(unsigned options) const
 
 		// take care on the numeric coefficient
 		ex coef = (possign? _ex1 : _ex_1);
-		if (m.overall_coeff.info(info_flags::positive)
-                                and not m.overall_coeff.is_integer_one())
+		if (m.overall_coeff.is_positive()
+                                and not m.overall_coeff.is_one())
 			prodseq.push_back(power(m.overall_coeff, exponent));
-		else if (m.overall_coeff.info(info_flags::negative) && m.overall_coeff != _ex_1)
+		else if (m.overall_coeff.is_negative()
+                         and not m.overall_coeff.is_minus_one())
 			prodseq.push_back(power(-m.overall_coeff, exponent));
 		else
 			coef *= m.overall_coeff;
@@ -1031,8 +1045,8 @@ ex power::expand(unsigned options) const
 			distrseq.push_back(power(expanded_basis, a.recombine_pair_to_ex(elem)));
 		
 		// Make sure that e.g. (x+y)^(2+a) expands the (x+y)^2 factor
-		if (ex_to<numeric>(a.overall_coeff).is_integer()) {
-			const numeric &num_exponent = ex_to<numeric>(a.overall_coeff);
+		if (a.overall_coeff.is_integer()) {
+			const numeric &num_exponent = a.overall_coeff;
 			int int_exponent = num_exponent.to_int();
 			if (int_exponent > 0 && is_exactly_a<add>(expanded_basis))
 				distrseq.push_back(expand_add(ex_to<add>(expanded_basis), int_exponent, options));
@@ -1346,8 +1360,10 @@ ex power::expand_add(const add & a, long n, unsigned options) const
 	// i.e. the number of unordered arrangements of m nonnegative integers
 	// which sum up to n.  It is frequently written as C_n(m) and directly
 	// related with binomial coefficients: binomial(n+m-1,m-1).
-	size_t result_size = binomial(numeric(n+a.nops()-1), numeric(a.nops()-1)).to_long();
-	if (!a.overall_coeff.is_zero()) {
+        long anops = a.nops() - 1;
+	long result_size = binomial(numeric(n + anops),
+                        numeric(anops)).to_long();
+	if (not a.overall_coeff.is_zero()) {
 		// the result's overall_coeff is one of the terms
 		--result_size;
 	}
@@ -1369,7 +1385,7 @@ ex power::expand_add(const add & a, long n, unsigned options) const
                         if (n == k)
                                 binomial_coefficient = *_num1_p;
                         else
-                                binomial_coefficient = binomial(numeric(n), numeric(k)) * (ex_to<numeric>(a.overall_coeff)).power(n-k);
+                                binomial_coefficient = binomial(numeric(n), numeric(k)) * a.overall_coeff.power(n-k);
 		}
 
 		// Multinomial expansion of power(+(x,...,z;0),k)*c^(n-k):
@@ -1423,7 +1439,7 @@ ex power::expand_add(const add & a, long n, unsigned options) const
 	if (a.overall_coeff.is_zero()) {
 		return dynallocate<add>(std::move(result)).setflag(status_flags::expanded).eval();
 	} else {
-		return dynallocate<add>(std::move(result), ex_to<numeric>(a.overall_coeff).power(n)).setflag(status_flags::expanded).eval();
+		return dynallocate<add>(std::move(result), a.overall_coeff.power(n)).setflag(status_flags::expanded).eval();
 	}
 }
 
@@ -1447,7 +1463,8 @@ ex power::expand_add_2(const add & a, unsigned options) const
 	for (auto cit0=a.seq.begin(); cit0!=last; ++cit0) {
 		const ex & r = cit0->rest;
 		const ex & c = cit0->coeff;
-		
+		const numeric& nc = ex_to<numeric>(c);
+
 		GINAC_ASSERT(!is_exactly_a<add>(r));
 		GINAC_ASSERT(!is_exactly_a<power>(r) ||
 		             !is_exactly_a<numeric>(ex_to<power>(r).exponent) ||
@@ -1456,7 +1473,7 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		             !is_exactly_a<mul>(ex_to<power>(r).basis) ||
 		             !is_exactly_a<power>(ex_to<power>(r).basis));
 		
-		if (c.is_equal(_ex1)) {
+		if (c.is_integer_one()) {
 			if (is_exactly_a<mul>(r)) {
 				result.push_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
 				                        _ex1));
@@ -1467,10 +1484,10 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		} else {
 			if (is_exactly_a<mul>(r)) {
 				result.push_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
-				                        ex_to<numeric>(c).power_dyn(*_num2_p)));
+				                        nc.pow_intexp(*_num2_p)));
 			} else {
 				result.push_back(expair(dynallocate<power>(r, _ex2),
-				                        ex_to<numeric>(c).power_dyn(*_num2_p)));
+				                        nc.pow_intexp(*_num2_p)));
 			}
 		}
 
@@ -1485,7 +1502,7 @@ ex power::expand_add_2(const add & a, unsigned options) const
 	// second part: add terms coming from overall_coeff (if != 0)
 	if (!a.overall_coeff.is_zero()) {
 		for (auto & i : a.seq)
-			result.push_back(a.combine_pair_with_coeff_to_pair(i, ex_to<numeric>(a.overall_coeff).mul_dyn(*_num2_p)));
+			result.push_back(a.combine_pair_with_coeff_to_pair(i, a.overall_coeff.mul(*_num2_p)));
 	}
 
 	GINAC_ASSERT(result.size() == result_size);
@@ -1493,7 +1510,7 @@ ex power::expand_add_2(const add & a, unsigned options) const
 	if (a.overall_coeff.is_zero()) {
 		return dynallocate<add>(std::move(result)).setflag(status_flags::expanded);
 	} else {
-		return dynallocate<add>(std::move(result), ex_to<numeric>(a.overall_coeff).power(2)).setflag(status_flags::expanded);
+		return dynallocate<add>(std::move(result), a.overall_coeff.pow_intexp(*_num2_p)).setflag(status_flags::expanded);
 	}
 }
 
@@ -1537,7 +1554,7 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 		distrseq.push_back(p);
 	}
 
-	const mul & result = static_cast<const mul &>((new mul(distrseq, ex_to<numeric>(m.overall_coeff).power_dyn(n)))->setflag(status_flags::dynallocated));
+	const mul & result = static_cast<const mul &>((new mul(distrseq, m.overall_coeff.pow_intexp(n)))->setflag(status_flags::dynallocated));
 	if (need_reexpand)
 		return ex(result).expand(options);
 	if (from_expand)
