@@ -66,24 +66,28 @@ symbol::symbol()
 symbol::symbol(std::string  initname, unsigned a_domain)
  : inherited(&symbol::tinfo_static), serial(next_serial++), name(std::move(initname)), TeX_name(default_TeX_name()), domain(a_domain), ret_type(return_types::commutative), ret_type_tinfo(&symbol::tinfo_static)
 {
+        set_domain(a_domain);
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
 
 symbol::symbol(std::string  initname, unsigned rt, tinfo_t rtt, unsigned a_domain)
  : inherited(&symbol::tinfo_static), serial(next_serial++), name(std::move(initname)), TeX_name(default_TeX_name()), domain(a_domain), ret_type(rt), ret_type_tinfo(rtt)
 {
+        set_domain(a_domain);
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
 
 symbol::symbol(std::string  initname, std::string  texname, unsigned a_domain)
  : inherited(&symbol::tinfo_static), serial(next_serial++), name(std::move(initname)), TeX_name(std::move(texname)), domain(a_domain), ret_type(return_types::commutative), ret_type_tinfo(&symbol::tinfo_static)
 {
+        set_domain(a_domain);
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
 
 symbol::symbol(std::string  initname, std::string  texname, unsigned rt, tinfo_t rtt, unsigned a_domain)
  : inherited(&symbol::tinfo_static), serial(next_serial++), name(std::move(initname)), TeX_name(std::move(texname)), domain(a_domain), ret_type(rt), ret_type_tinfo(rtt)
 {
+        set_domain(a_domain);
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
 
@@ -103,6 +107,7 @@ symbol::symbol(const archive_node &n, lst &sym_lst)
 		domain = domain::complex;
 	if (!n.find_unsigned("return_type", ret_type))
 		ret_type = return_types::commutative;
+        set_domain(domain);
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
 
@@ -175,6 +180,7 @@ void symbol::set_domain(unsigned d)
                 case domain::complex:
                         iflags.set(info_flags::real, false);
                         iflags.set(info_flags::positive, false);
+                        iflags.set(info_flags::negative, false);
                         iflags.set(info_flags::nonnegative, false);
                         iflags.set(info_flags::integer, false);
                         iflags.set(info_flags::even, false);
@@ -189,6 +195,14 @@ void symbol::set_domain(unsigned d)
                                 iflags.set(info_flags::positive, false);
                                 iflags.set(info_flags::nonnegative, false);
                         }
+                        if(iflags.get(info_flags::negative)) {
+                                iflags.set(info_flags::negative, true);
+                                iflags.set(info_flags::nonnegative, false);
+                        }  
+                        else {
+                                iflags.set(info_flags::negative, false);
+                                iflags.set(info_flags::nonnegative, false);
+                        }
                         if(iflags.get(info_flags::integer)) {
                                 iflags.set(info_flags::integer, true);
                         }
@@ -205,6 +219,7 @@ void symbol::set_domain(unsigned d)
                 case domain::positive:
                         iflags.set(info_flags::real, true);
                         iflags.set(info_flags::positive, true);
+                        iflags.set(info_flags::negative, false);
                         iflags.set(info_flags::nonnegative, true);
                         if(iflags.get(info_flags::integer)) {
                                 iflags.set(info_flags::integer, true);
@@ -219,6 +234,24 @@ void symbol::set_domain(unsigned d)
                                 iflags.set(info_flags::even, false);
                         }
                         break;
+                case domain::negative:
+                        iflags.set(info_flags::real, true);
+                        iflags.set(info_flags::negative, true);
+                        iflags.set(info_flags::positive, false);
+                        iflags.set(info_flags::nonnegative, false);
+                        if(iflags.get(info_flags::integer)) {
+                                iflags.set(info_flags::integer, true);
+                        }
+                        else {
+                                iflags.set(info_flags::integer, false);
+                        }
+                        if(iflags.get(info_flags::even)) {
+                                iflags.set(info_flags::even, true);
+                        }
+                        else {
+                                iflags.set(info_flags::even, false);
+                        }
+                        break;
                 case domain::integer:
                         iflags.set(info_flags::real, true);
                         iflags.set(info_flags::integer, true);
@@ -228,6 +261,12 @@ void symbol::set_domain(unsigned d)
                         else {
                                 iflags.set(info_flags::positive, false);
                         }
+                        if(iflags.get(info_flags::negative)) {
+                                iflags.set(info_flags::negative, true);
+                        }
+                        else {
+                                iflags.set(info_flags::negative, false);
+                        }
                         if(iflags.get(info_flags::even)) {
                                 iflags.set(info_flags::even, true);
                         }
@@ -242,6 +281,12 @@ void symbol::set_domain(unsigned d)
                         else {
                                 iflags.set(info_flags::positive, false);
                         }
+                        if(iflags.get(info_flags::negative)) {
+                                iflags.set(info_flags::negative, true);
+                        }
+                        else {
+                                iflags.set(info_flags::negative, false);
+                        }
                         iflags.set(info_flags::integer, true);
                         iflags.set(info_flags::even, true);
                         break;
